/*
 * clamTheme.c,v 1.6 2004/07/01 15:57:51 jenglish Exp
 *
 * Copyright (C) 2004 Joe English
 *
 * Tile widget set: another theme engine.
 * Inspired by the XFCE family of Gnome themes.
 *
 * TODO: Scale and Notebook widgets.
 */

#include <tk.h>
#include "tkTheme.h"

#define STR(x) StR(x)
#define StR(x) #x

#define SCROLLBAR_THICKNESS 14

#define FRAME_COLOR	"#dcdad5"
#define DARK_COLOR  	"#cfcdc8"
#define DARKER_COLOR 	"#bab5ab"
#define DARKEST_COLOR	"#9e9a91"

/*------------------------------------------------------------------------
 * +++ Utilities.
 */

static void DrawSmoothBorder(
    Tk_Window tkwin, Drawable d, TTK_Box b,
    Tcl_Obj *outerColorObj, Tcl_Obj *upperColorObj, Tcl_Obj *lowerColorObj)
{
    Display *display = Tk_Display(tkwin);
    int x1 = b.x, x2 = b.x + b.width - 1;
    int y1 = b.y, y2 = b.y + b.height - 1;
    GC gc;

    if (   outerColorObj
	&& (gc=Tk_GCForColor(Tk_GetColorFromObj(tkwin,outerColorObj),d)))
    {
	XDrawLine(display,d,gc, x1+1,y1, x2-1,y1); /* N */
	XDrawLine(display,d,gc, x1+1,y2, x2-1,y2); /* S */
	XDrawLine(display,d,gc, x1,y1+1, x1,y2-1); /* E */
	XDrawLine(display,d,gc, x2,y1+1, x2,y2-1); /* W */
    }

    if (   upperColorObj
	&& (gc=Tk_GCForColor(Tk_GetColorFromObj(tkwin,upperColorObj),d)))
    {
	XDrawLine(display,d,gc, x1+1,y1+1, x2-1,y1+1); /* N */
	XDrawLine(display,d,gc, x1+1,y1+1, x1+1,y2-1); /* E */
    }

    if (   lowerColorObj
	&& (gc=Tk_GCForColor(Tk_GetColorFromObj(tkwin,lowerColorObj),d)))
    {
	XDrawLine(display,d,gc, x2-1,y2-1, x1+1,y2-1); /* S */
	XDrawLine(display,d,gc, x2-1,y2-1, x2-1,y1+1); /* W */
    }
}

static GC BackgroundGC(Tk_Window tkwin, Tcl_Obj *backgroundObj)
{
    Tk_3DBorder bd = Tk_Get3DBorderFromObj(tkwin, backgroundObj);
    return Tk_3DBorderGC(tkwin, bd, TK_3D_FLAT_GC);
}

/*------------------------------------------------------------------------
 * +++ Border element.
 */

typedef struct {
    Tcl_Obj 	*borderColorObj;
    Tcl_Obj 	*lightColorObj;
    Tcl_Obj 	*darkColorObj;
    Tcl_Obj 	*reliefObj;
} BorderElement;

static TTK_ElementOptionSpec BorderElementOptions[] = {
    { "-bordercolor", TK_OPTION_COLOR,
	Tk_Offset(BorderElement,borderColorObj), DARKEST_COLOR },
    { "-lightcolor", TK_OPTION_COLOR,
	Tk_Offset(BorderElement,lightColorObj), "#ffffff" },
    { "-darkcolor", TK_OPTION_COLOR,
	Tk_Offset(BorderElement,darkColorObj), DARK_COLOR },
    { "-relief", TK_OPTION_RELIEF,
	Tk_Offset(BorderElement,reliefObj), "flat" },
    {0,0,0}
};

static void BorderElementGeometry(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    *paddingPtr = TTK_UniformPadding(2);
}

static void BorderElementDraw(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    Drawable d, TTK_Box b, unsigned state)
{
    BorderElement *border = elementRecord;
    int relief = TK_RELIEF_FLAT;
    Tcl_Obj *outer = 0, *upper = 0, *lower = 0;

    Tk_GetReliefFromObj(NULL, border->reliefObj, &relief);

    switch (relief) {
	case TK_RELIEF_GROOVE :
	case TK_RELIEF_RIDGE :
	case TK_RELIEF_RAISED :
	    outer = border->borderColorObj;
	    upper = border->lightColorObj;
	    lower = border->darkColorObj;
	    break;
	case TK_RELIEF_SUNKEN :
	    outer = border->borderColorObj;
	    upper = border->darkColorObj;
	    lower = border->lightColorObj;
	    break;
	case TK_RELIEF_FLAT :
	    outer = upper = lower = 0;
	    break;
	case TK_RELIEF_SOLID :
	    outer = upper = lower = border->borderColorObj;
	    break;
    }

    DrawSmoothBorder(tkwin, d, b, outer, upper, lower);
}

static TTK_ElementSpec BorderElementSpec = {
    TK_STYLE_VERSION_2,
    sizeof(BorderElement),
    BorderElementOptions,
    BorderElementGeometry,
    BorderElementDraw
};

/*------------------------------------------------------------------------
 * +++ Field element.
 */

typedef struct {
    Tcl_Obj 	*borderColorObj;
    Tcl_Obj 	*lightColorObj;
    Tcl_Obj 	*darkColorObj;
    Tcl_Obj 	*backgroundObj;
} FieldElement;

static TTK_ElementOptionSpec FieldElementOptions[] = {
    { "-bordercolor", TK_OPTION_COLOR,
	Tk_Offset(FieldElement,borderColorObj), DARKEST_COLOR },
    { "-lightcolor", TK_OPTION_COLOR,
	Tk_Offset(FieldElement,lightColorObj), "#ffffff" },
    { "-darkcolor", TK_OPTION_COLOR,
	Tk_Offset(FieldElement,darkColorObj), DARK_COLOR },
    { "-fieldbackground", TK_OPTION_BORDER,
	Tk_Offset(FieldElement,backgroundObj), "white" },
    {0,0,0}
};

static void FieldElementGeometry(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    *paddingPtr = TTK_UniformPadding(2);
}

static void FieldElementDraw(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    Drawable d, TTK_Box b, unsigned state)
{
    FieldElement *field = elementRecord;
    Tk_3DBorder bg = Tk_Get3DBorderFromObj(tkwin, field->backgroundObj);
    TTK_Box f = TTK_PadBox(b, TTK_UniformPadding(2));
    Tcl_Obj *outer = field->borderColorObj,
	    *upper = field->darkColorObj,
	    *lower = field->lightColorObj;

    DrawSmoothBorder(tkwin, d, b, outer, upper, lower);
    Tk_Fill3DRectangle(
	tkwin, d, bg, f.x, f.y, f.width, f.height, 0, TK_RELIEF_SUNKEN);
}

static TTK_ElementSpec FieldElementSpec = {
    TK_STYLE_VERSION_2,
    sizeof(FieldElement),
    FieldElementOptions,
    FieldElementGeometry,
    FieldElementDraw
};

/*------------------------------------------------------------------------
 * +++ Indicator elements for check and radio buttons.
 */

typedef struct {
    Tcl_Obj *sizeObj;
    Tcl_Obj *marginObj;
    Tcl_Obj *backgroundObj;
    Tcl_Obj *foregroundObj;
    Tcl_Obj *upperColorObj;
    Tcl_Obj *lowerColorObj;
} IndicatorElement;

static TTK_ElementOptionSpec IndicatorElementOptions[] = {
    { "-indicatorsize", TK_OPTION_PIXELS,
	Tk_Offset(IndicatorElement,sizeObj), "10" },
    { "-indicatormargin", TK_OPTION_STRING,
	Tk_Offset(IndicatorElement,marginObj), "1" },
    { "-indicatorbackground", TK_OPTION_COLOR,
	Tk_Offset(IndicatorElement,backgroundObj), "white" },
    { "-indicatorforeground", TK_OPTION_COLOR,
	Tk_Offset(IndicatorElement,foregroundObj), "black" },
    { "-upperbordercolor", TK_OPTION_COLOR,
	Tk_Offset(IndicatorElement,upperColorObj), DARKEST_COLOR },
    { "-lowerbordercolor", TK_OPTION_COLOR,
	Tk_Offset(IndicatorElement,lowerColorObj), DARK_COLOR },
    {0,0,0}
};

static void
IndicatorElementGeometry(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    IndicatorElement *indicator = elementRecord;
    int size = 10;
    TTK_GetPaddingFromObj(NULL, tkwin, indicator->marginObj, paddingPtr);
    Tk_GetPixelsFromObj(NULL, tkwin, indicator->sizeObj, &size);
    *widthPtr = *heightPtr = size;
}

static void
RadioIndicatorElementDraw(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    Drawable d, TTK_Box b, unsigned state)
{
    IndicatorElement *indicator = elementRecord;
    GC gcb=Tk_GCForColor(Tk_GetColorFromObj(tkwin,indicator->backgroundObj),d);
    GC gcf=Tk_GCForColor(Tk_GetColorFromObj(tkwin,indicator->foregroundObj),d);
    GC gcu=Tk_GCForColor(Tk_GetColorFromObj(tkwin,indicator->upperColorObj),d);
    GC gcl=Tk_GCForColor(Tk_GetColorFromObj(tkwin,indicator->lowerColorObj),d);
    TTK_Padding padding;

    TTK_GetPaddingFromObj(NULL, tkwin, indicator->marginObj, &padding);
    b = TTK_PadBox(b, padding);

    XFillArc(Tk_Display(tkwin),d,gcb, b.x,b.y,b.width,b.height, 0,360*64);
    XDrawArc(Tk_Display(tkwin),d,gcl, b.x,b.y,b.width,b.height, 225*64,180*64);
    XDrawArc(Tk_Display(tkwin),d,gcu, b.x,b.y,b.width,b.height, 45*64,180*64);

    if (state & TTK_STATE_SELECTED) {
	b = TTK_PadBox(b,TTK_UniformPadding(3));
	XFillArc(Tk_Display(tkwin),d,gcf, b.x,b.y,b.width,b.height, 0,360*64);
	XDrawArc(Tk_Display(tkwin),d,gcf, b.x,b.y,b.width,b.height, 0,360*64);
    }
}

static void
CheckIndicatorElementDraw(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    Drawable d, TTK_Box b, unsigned state)
{
    Display *display = Tk_Display(tkwin);
    IndicatorElement *indicator = elementRecord;
    GC gcb=Tk_GCForColor(Tk_GetColorFromObj(tkwin,indicator->backgroundObj),d);
    GC gcf=Tk_GCForColor(Tk_GetColorFromObj(tkwin,indicator->foregroundObj),d);
    GC gcu=Tk_GCForColor(Tk_GetColorFromObj(tkwin,indicator->upperColorObj),d);
    GC gcl=Tk_GCForColor(Tk_GetColorFromObj(tkwin,indicator->lowerColorObj),d);
    TTK_Padding padding;

    TTK_GetPaddingFromObj(NULL, tkwin, indicator->marginObj, &padding);
    b = TTK_PadBox(b, padding);

    XFillRectangle(display,d,gcb, b.x,b.y,b.width,b.height);
    XDrawLine(display,d,gcl, b.x,b.y+b.height, b.x+b.width,b.y+b.height);
    XDrawLine(display,d,gcl, b.x+b.width,b.y,  b.x+b.width,b.y+b.height);
    XDrawLine(display,d,gcu, b.x,b.y, b.x,b.y+b.height);
    XDrawLine(display,d,gcu, b.x,b.y, b.x+b.width,b.y);

    if (state & TTK_STATE_SELECTED) {
	b = TTK_PadBox(b,TTK_UniformPadding(2));

	XDrawLine(display,d,gcf, b.x, b.y, b.x+b.width, b.y+b.height);
	XDrawLine(display,d,gcf, b.x+1, b.y, b.x+b.width, b.y+b.height-1);
	XDrawLine(display,d,gcf, b.x, b.y+1, b.x+b.width-1, b.y+b.height);

	XDrawLine(display,d,gcf,
	b.x+b.width, b.y,
	b.x, b.y+b.height);
	XDrawLine(display,d,gcf, b.x+1, b.y+b.height, b.x+b.width, b.y+1);
	XDrawLine(display,d,gcf, b.x, b.y+b.height-1, b.x+b.width-1, b.y);
    }
}

static TTK_ElementSpec RadioIndicatorElementSpec = {
    TK_STYLE_VERSION_2,
    sizeof(IndicatorElement),
    IndicatorElementOptions,
    IndicatorElementGeometry,
    RadioIndicatorElementDraw
};

static TTK_ElementSpec CheckIndicatorElementSpec = {
    TK_STYLE_VERSION_2,
    sizeof(IndicatorElement),
    IndicatorElementOptions,
    IndicatorElementGeometry,
    CheckIndicatorElementDraw
};

/*------------------------------------------------------------------------
 * +++ Scrollbar elements: trough, arrows, thumb.
 *
 * Notice that the trough element has 0 internal padding;
 * that way the thumb and arrow borders overlap the trough.
 */

typedef struct { /* Common element record for scrollbar elements */
    Tcl_Obj 	*orientObj;
    Tcl_Obj 	*backgroundObj;
    Tcl_Obj 	*borderColorObj;
    Tcl_Obj 	*troughColorObj;
    Tcl_Obj 	*lightColorObj;
    Tcl_Obj 	*darkColorObj;
    Tcl_Obj 	*arrowColorObj;
    Tcl_Obj 	*arrowSizeObj;
    Tcl_Obj 	*gripCountObj;
} ScrollbarElement;

static TTK_ElementOptionSpec ScrollbarElementOptions[] = {
    { "-orient", TK_OPTION_ANY,
	Tk_Offset(ScrollbarElement, orientObj), "horizontal" },
    { "-background", TK_OPTION_BORDER,
	Tk_Offset(ScrollbarElement,backgroundObj), FRAME_COLOR },
    { "-bordercolor", TK_OPTION_COLOR,
	Tk_Offset(ScrollbarElement,borderColorObj), DARKEST_COLOR },
    { "-troughcolor", TK_OPTION_COLOR,
	Tk_Offset(ScrollbarElement,troughColorObj), DARKER_COLOR },
    { "-lightcolor", TK_OPTION_COLOR,
	Tk_Offset(ScrollbarElement,lightColorObj), "#ffffff" },
    { "-darkcolor", TK_OPTION_COLOR,
	Tk_Offset(ScrollbarElement,darkColorObj), DARK_COLOR },
    { "-arrowcolor", TK_OPTION_COLOR,
	Tk_Offset(ScrollbarElement,arrowColorObj), "#000000" },
    { "-arrowsize", TK_OPTION_PIXELS,
	Tk_Offset(ScrollbarElement,arrowSizeObj), STR(SCROLLBAR_THICKNESS) },
    { "-gripcount", TK_OPTION_INT,
	Tk_Offset(ScrollbarElement,gripCountObj), "5" },
    {0,0,0}
};

static void TroughElementDraw(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    Drawable d, TTK_Box b, unsigned state)
{
    ScrollbarElement *sb = elementRecord;
    GC gcb = Tk_GCForColor(Tk_GetColorFromObj(tkwin,sb->borderColorObj),d);
    GC gct = Tk_GCForColor(Tk_GetColorFromObj(tkwin,sb->troughColorObj),d);
    XFillRectangle(Tk_Display(tkwin), d, gct, b.x, b.y, b.width-1, b.height-1);
    XDrawRectangle(Tk_Display(tkwin), d, gcb, b.x, b.y, b.width-1, b.height-1);
}

static TTK_ElementSpec TroughElementSpec = {
    TK_STYLE_VERSION_2,
    sizeof(ScrollbarElement),
    ScrollbarElementOptions,
    NullElementGeometry,
    TroughElementDraw
};

static void ThumbElementGeometry(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    ScrollbarElement *sb = elementRecord;
    int size = SCROLLBAR_THICKNESS;
    Tcl_GetIntFromObj(NULL, sb->arrowSizeObj, &size);
    *widthPtr = *heightPtr = size;
}

static void ThumbElementDraw(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    Drawable d, TTK_Box b, unsigned state)
{
    ScrollbarElement *sb = elementRecord;
    int gripCount = 3, orient = TTK_ORIENT_HORIZONTAL;
    GC lightGC, darkGC;
    int x1, y1, x2, y2, dx, dy, i;

    DrawSmoothBorder(tkwin, d, b,
	sb->borderColorObj, sb->lightColorObj, sb->darkColorObj);
    XFillRectangle(
	Tk_Display(tkwin), d, BackgroundGC(tkwin, sb->backgroundObj),
	b.x+2, b.y+2, b.width-4, b.height-4);

    /*
     * Draw grip:
     */
    TTK_GetOrientationFromObj(NULL, sb->orientObj, &orient);
    Tcl_GetIntFromObj(NULL, sb->gripCountObj, &gripCount);
    lightGC = Tk_GCForColor(Tk_GetColorFromObj(tkwin,sb->lightColorObj),d);
    darkGC = Tk_GCForColor(Tk_GetColorFromObj(tkwin,sb->borderColorObj),d);

    if (orient == TTK_ORIENT_HORIZONTAL) {
	dx = 1; dy = 0;
	x1 = x2 = b.x + b.width / 2 - gripCount;
	y1 = b.y + 2;
	y2 = b.y + b.height - 3;
    } else {
	dx = 0; dy = 1;
	y1 = y2 = b.y + b.height / 2 - gripCount;
	x1 = b.x + 2;
	x2 = b.x + b.width - 3;
    }

    for (i=0; i<gripCount; ++i) {
	XDrawLine(Tk_Display(tkwin), d, darkGC, x1,y1, x2,y2);
	x1 += dx; x2 += dx; y1 += dy; y2 += dy;
	XDrawLine(Tk_Display(tkwin), d, lightGC, x1,y1, x2,y2);
	x1 += dx; x2 += dx; y1 += dy; y2 += dy;
    }
}

static TTK_ElementSpec ThumbElementSpec = {
    TK_STYLE_VERSION_2,
    sizeof(ScrollbarElement),
    ScrollbarElementOptions,
    ThumbElementGeometry,
    ThumbElementDraw
};

/*
 * Scrollbar arrows:
 */
static int ArrowElements[] = { ARROW_UP, ARROW_DOWN, ARROW_LEFT, ARROW_RIGHT };

static void ArrowElementGeometry(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    ScrollbarElement *sb = elementRecord;
    int size = SCROLLBAR_THICKNESS;
    Tcl_GetIntFromObj(NULL, sb->arrowSizeObj, &size);
    *widthPtr = *heightPtr = size;
}

static void ArrowElementDraw(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    Drawable d, TTK_Box b, unsigned state)
{
    ArrowDirection dir = *(ArrowDirection*)clientData;
    ScrollbarElement *sb = elementRecord;
    GC gc = Tk_GCForColor(Tk_GetColorFromObj(tkwin,sb->arrowColorObj), d);
    int h, cx, cy;

    DrawSmoothBorder(tkwin, d, b,
	sb->borderColorObj, sb->lightColorObj, sb->darkColorObj);

    XFillRectangle(
	Tk_Display(tkwin), d, BackgroundGC(tkwin, sb->backgroundObj),
	b.x+2, b.y+2, b.width-4, b.height-4);

    b = TTK_PadBox(b, TTK_UniformPadding(3));
    h = b.width < b.height ? b.width : b.height;
    ArrowSize(h/2, dir, &cx, &cy);
    b = TTK_AnchorBox(b, cx, cy, TK_ANCHOR_CENTER);

    DrawArrow(Tk_Display(tkwin), d, gc, b, dir);
}


static TTK_ElementSpec ArrowElementSpec = {
    TK_STYLE_VERSION_2,
    sizeof(ScrollbarElement),
    ScrollbarElementOptions,
    ArrowElementGeometry,
    ArrowElementDraw
};

/*------------------------------------------------------------------------
 * +++ Notebook elements.
 */

typedef struct {
    Tcl_Obj *backgroundObj;
    Tcl_Obj *borderColorObj;
    Tcl_Obj *lightColorObj;
    Tcl_Obj *darkColorObj;
} NotebookElement;

static TTK_ElementOptionSpec NotebookElementOptions[] = {
    { "-background", TK_OPTION_BORDER,
	Tk_Offset(NotebookElement,backgroundObj), FRAME_COLOR },
    { "-bordercolor", TK_OPTION_COLOR,
	Tk_Offset(NotebookElement,borderColorObj), DARKEST_COLOR },
    { "-lightcolor", TK_OPTION_COLOR,
	Tk_Offset(NotebookElement,lightColorObj), "#ffffff" },
    { "-darkcolor", TK_OPTION_COLOR,
	Tk_Offset(NotebookElement,darkColorObj), DARK_COLOR },
    {0,0,0}
};

static void TabElementGeometry(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    int borderWidth = 2;
    paddingPtr->top = paddingPtr->left = paddingPtr->right = borderWidth;
    paddingPtr->bottom = 0;
}

static void TabElementDraw(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    Drawable d, TTK_Box b, unsigned int state)
{
    NotebookElement *tab = elementRecord;
    Tk_3DBorder border = Tk_Get3DBorderFromObj(tkwin, tab->backgroundObj);
    Display *display = Tk_Display(tkwin);
    int borderWidth = 2, dh = 0;
    int x1,y1,x2,y2;
    GC gc;

    if (state & TTK_STATE_SELECTED) {
	dh = borderWidth;
    }

    Tk_Fill3DRectangle(tkwin, d, border,
	b.x+2, b.y+2, b.width-1, b.height-2+dh, borderWidth, TK_RELIEF_FLAT);

    x1 = b.x, x2 = b.x + b.width;
    y1 = b.y, y2 = b.y + b.height;

    gc=Tk_GCForColor(Tk_GetColorFromObj(tkwin,tab->borderColorObj),d);
    XDrawLine(display,d,gc, x1,y1+1, x1,y2);
    XDrawLine(display,d,gc, x2,y1+1, x2,y2);
    XDrawLine(display,d,gc, x1+1,y1, x2-1,y1);

    gc=Tk_GCForColor(Tk_GetColorFromObj(tkwin,tab->lightColorObj),d);
    XDrawLine(display,d,gc, x1+1,y1+1, x1+1,y2-1+dh);
    XDrawLine(display,d,gc, x1+1,y1+1, x2-1,y1+1);

}

static TTK_ElementSpec TabElementSpec =
{
    TK_STYLE_VERSION_2,
    sizeof(NotebookElement),
    NotebookElementOptions,
    TabElementGeometry,
    TabElementDraw
};

static void ClientElementGeometry(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    int borderWidth = 2;
    *paddingPtr = TTK_UniformPadding(borderWidth);
}

static void ClientElementDraw(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    Drawable d, TTK_Box b, unsigned int state)
{
    NotebookElement *ce = elementRecord;
    Tk_3DBorder border = Tk_Get3DBorderFromObj(tkwin, ce->backgroundObj);
    int borderWidth = 2;

    Tk_Fill3DRectangle(tkwin, d, border,
	b.x, b.y, b.width, b.height, borderWidth,TK_RELIEF_FLAT);
    DrawSmoothBorder(tkwin, d, b,
    	ce->borderColorObj, ce->lightColorObj, ce->darkColorObj);
}

static TTK_ElementSpec ClientElementSpec =
{
    TK_STYLE_VERSION_2,
    sizeof(NotebookElement),
    NotebookElementOptions,
    ClientElementGeometry,
    ClientElementDraw
};

/*------------------------------------------------------------------------
 * +++ Modified widget layouts.
 */

TTK_BEGIN_LAYOUT(ComboboxLayout)
    TTK_NODE("Combobox.background", TTK_FILL_BOTH)
    TTK_NODE("Combobox.downarrow", TTK_PACK_RIGHT|TTK_FILL_Y)
    TTK_GROUP("Combobox.field", TTK_PACK_LEFT|TTK_FILL_BOTH|TTK_EXPAND,
	TTK_GROUP("Combobox.padding", TTK_FILL_BOTH|TTK_EXPAND,
	    TTK_NODE("Combobox.textarea", TTK_FILL_BOTH)))
TTK_END_LAYOUT

/*------------------------------------------------------------------------
 * +++ Initialization.
 */

int DLLEXPORT
ClamTheme_Init(Tcl_Interp *interp)
{
    TTK_Theme themePtr, parentPtr;

    parentPtr = TTK_GetTheme(interp, "alt");
    themePtr = TTK_CreateTheme(interp, "clam", parentPtr);

    if (!themePtr)
        return TCL_ERROR;

    TTK_RegisterElementSpec(
	themePtr, "border", &BorderElementSpec, NULL);
    TTK_RegisterElementSpec(
	themePtr, "field", &FieldElementSpec, NULL);
    TTK_RegisterElementSpec(
	themePtr, "Scrollbar.trough", &TroughElementSpec, NULL);
    TTK_RegisterElementSpec(
	themePtr, "Scrollbar.thumb", &ThumbElementSpec, NULL);
    TTK_RegisterElementSpec(
	themePtr, "uparrow", &ArrowElementSpec, &ArrowElements[0]);
    TTK_RegisterElementSpec(
	themePtr, "downarrow", &ArrowElementSpec, &ArrowElements[1]);
    TTK_RegisterElementSpec(
	themePtr, "leftarrow", &ArrowElementSpec, &ArrowElements[2]);
    TTK_RegisterElementSpec(
	themePtr, "rightarrow", &ArrowElementSpec, &ArrowElements[3]);

    TTK_RegisterElementSpec(
	themePtr, "Radiobutton.indicator", &RadioIndicatorElementSpec, NULL);
    TTK_RegisterElementSpec(
	themePtr, "Checkbutton.indicator", &CheckIndicatorElementSpec, NULL);

    TTK_RegisterElementSpec(themePtr, "tab", &TabElementSpec, NULL);
    TTK_RegisterElementSpec(themePtr, "client", &ClientElementSpec, NULL);

    TTK_RegisterLayout(themePtr, "TCombobox", ComboboxLayout);

    return TCL_OK;
}
