/* pixmapTheme.c - Copyright (C) 2004 Pat Thoyts <patthoyts@users.sf.net>
 *
 * pixmapTheme.c,v 1.20 2004/03/09 16:44:40 jenglish Exp
 */

#include <string.h>
#include <tk.h>
#include "tkTheme.h"

#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif

/* ---------------------------------------------------------------------- */

typedef enum {TILING_TILE, TILING_FIXED, TILING_STRETCH } TILING;
const char *tilingStrings[] = {
    "tile", "fixed", "stretch", NULL
};

enum {BACKGROUND_TRANSPARENT, BACKGROUND_OPAQUE };
const char *backgroundStrings[] = {
    "transparent", "opaque", NULL
};

/* 
 * ----------------------------------------------------------------------
 *
 * Fill the destination area of the drawable from the source area.
 */

static void 
TTK_Tile(Tk_Window tkwin, Drawable d, Tk_Image image, 
	 TTK_Box *src, TTK_Box *dst, int border)
{
    TTK_Box core;
    int x, y, xx, yy, sx, sy, cw, ch;

    core.x = src->x + border;
    core.y = src->y + border;
    core.width = src->width-(border*2);
    core.height = src->height-(border*2);

    /* dst limits -- be careful not to overdraw. PNG's may have TRANSPARENT parts */
    xx = dst->x + dst->width - border;
    yy = dst->y + dst->height - border;

    x = dst->x;
    y = dst->y;
    cw = MIN(core.width+border,dst->width - border);
    ch = MIN(core.height+border,dst->height - border);

    /* top row */
    Tk_RedrawImage(image, src->x, src->y, cw, ch, d, x, y);
    for (x += cw; x < xx; x += core.width) {
	cw = MIN(core.width,xx-x);
	Tk_RedrawImage(image, core.x, src->y, cw, ch, d, x, y);
    }
    y = y + ch;

    /* tile the rest of the image (first column includes the left edge) */
    if (yy - y > core.height) {
        ch = MIN(core.height,yy-y);
	for (; y < yy; y += ch) {
	    x = dst->x;
	    cw = MIN(core.width+border,xx-x);
	    ch = MIN(core.height,yy-y);
	    Tk_RedrawImage(image, src->x, core.y, cw, ch, d, x, y);

	    for (x += cw; x < xx; x += cw) {
		cw = MIN(core.width, xx-x);
		Tk_RedrawImage(image, core.x, core.y, cw, ch, d, x, y);
	    }
	}
    }

    /* Bottom edge */
    x = dst->x;
    y -= border;
    ch = dst->height - (y - dst->y);
    ch = MAX(ch, border);
    cw = MIN(core.width+border,dst->width-border);
    sy = src->y + src->height - ch;
    Tk_RedrawImage(image, src->x, sy, cw, ch, d, x, y);
    for (x += cw; x < xx; x += cw) {
	cw = MIN(core.width,xx-x);
	Tk_RedrawImage(image, core.x, sy, cw, ch, d, x, y);
    }

    /* Right edge */
    x = dst->x + dst->width - border;
    y = dst->y;
    ch = MIN(core.height+border,(dst->height-border));
    sx = src->x + src->width - border;
    Tk_RedrawImage(image, sx, src->y, border, ch, d, x, y);
    for (y += ch; y < yy; y += ch) {
	ch = MIN(core.height,yy-y);
	Tk_RedrawImage(image, sx, core.y, border, ch, d, x, y);
    }

    /* The final corner */
    x  = dst->x + dst->width - border;
    y  = dst->y + dst->height - border;
    sx = src->x + src->width - border;
    sy = src->y + src->height - border;
    Tk_RedrawImage(image, sx, sy, border, border, d, x, y);
}

#if UNUSED
/*
 * Check the scale factors. If necessary, multiply the center to create a 
 * scaled image. Then call the Tile proc to paint it.
 */
static void 
TTK_Stretch(Tk_Window tkwin, Drawable d, Tk_Image image, 
	    TTK_Box *src, TTK_Box *dst, int border)
{
    int xscale, yscale;
    xscale = (int)(((double)dst->width / (double)src->width) + 0.5);
    yscale = (int)(((double)dst->height / (double)src->height) + 0.5);
    if (xscale > 1 || yscale > 1) {
    }

}
#endif

/* ---------------------------------------------------------------------- */
typedef struct
{
    int cx;
    int cy;
} TTK_Size;

static int TTK_GetSizeFromObj(Tcl_Interp *interp, Tk_Window tkwin,
    Tcl_Obj *objPtr, TTK_Size *sizePtr)
{
    Tcl_Obj **elts;
    int n_elts, r = TCL_OK;
    
    if (objPtr) {
	r = Tcl_ListObjGetElements(interp, objPtr, &n_elts, &elts);
	if (r == TCL_OK) {
	    if (n_elts != 2) {
		Tcl_SetObjResult(interp, 
		    Tcl_NewStringObj("invalid size specification: must be {cx cy}", -1));
		r = TCL_ERROR;
	    }
	}

	if (r == TCL_OK)
	    r = Tk_GetPixelsFromObj(interp, tkwin, elts[0], &sizePtr->cx);
	if (r == TCL_OK)
	    r = Tk_GetPixelsFromObj(interp, tkwin, elts[1], &sizePtr->cy);
    } else {
	sizePtr->cx = 0;
	sizePtr->cy = 0;
    }

    return r;
}

/*
 * ClientData for pixmap elements:
 */
typedef struct {
    Tcl_Interp *interp;		/* Interpreter to look up images */
    TTK_ResourceCache cache;	/* Resource cache for images */
    TTK_StateMap imageMap;	/* state-based lookup table for images */
    TILING tiling;		/* how to tile the image */
    int border;			/* image border (uniform) */
    int background;		/* transparent / opaque */
    Tcl_Obj *minsizeObj;        /* minimum size ({0,0} == no minimum) */
} PixmapData;

static Tk_OptionSpec PixmapOptionSpecs[] = 
{
    { TK_OPTION_STRING_TABLE, "-tiling", "tiling", "Tiling", "fixed",
	-1, Tk_Offset(PixmapData, tiling), 
	0, (ClientData)tilingStrings, 0 },
    { TK_OPTION_STRING, "-images", "images", "Images", "",
	Tk_Offset(PixmapData,imageMap), -1,
	0,0,0 },
    { TK_OPTION_PIXELS, "-border", "border", "Border", "0",
	-1, Tk_Offset(PixmapData,border),
	0,0,0 },
    { TK_OPTION_STRING_TABLE, "-background", "background", "Background",
	"transparent", -1, Tk_Offset(PixmapData, background),
	0, (ClientData)backgroundStrings, 0},
    { TK_OPTION_STRING, "-minsize", "minsize", "Minsize",
	"0", Tk_Offset(PixmapData, minsizeObj), -1,
	0, (ClientData)NULL, 0},
    { TK_OPTION_END }
};

/*
 * Element record for pixmap elements:
 */
typedef struct {
    Tcl_Obj *backgroundObj;
} PixmapElement;

static TTK_ElementOptionSpec PixmapElementOptions[] =
{
    { "-background", TK_OPTION_BORDER,
	Tk_Offset(PixmapElement,backgroundObj), DEFAULT_BACKGROUND },
    {NULL}
};

static void
PixmapElementGeometry(
    void *clientData, void *elementRecord,
    Tk_Window tkwin, int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    PixmapData *pixmapData = clientData;
    TTK_Size minsize = {0,0};
    Tcl_Obj *imageObj = TTK_StateMapLookup(NULL, pixmapData->imageMap, 0);

    if (pixmapData->minsizeObj) {
	TTK_GetSizeFromObj(NULL, tkwin, pixmapData->minsizeObj, &minsize);
    }

    if (imageObj) {
	Tk_Image image = TTK_UseImage(pixmapData->cache, tkwin, imageObj);
	if (image) {
	    Tk_SizeOfImage(image, widthPtr, heightPtr);
	    if (pixmapData->tiling != TILING_FIXED) {
		if (minsize.cx != 0)
		    *widthPtr = minsize.cx;
		if (minsize.cy != 0)
		    *heightPtr = minsize.cy;
	    }	    
	}

	/*
	 * The point of this is to prevent the border region of the image
	 * from being overwritten by child elements. For instance, the 
	 * Progress.trough needs to protect its border from the Progress.bar
	 * element. Doing it like this avoids changing size of this element.
	 */
        paddingPtr->left = paddingPtr->right = paddingPtr->top = paddingPtr->bottom
	    = pixmapData->border;
	*widthPtr -= (2 * pixmapData->border);
	*heightPtr -= (2 * pixmapData->border);
    }
}

static void
PixmapElementDraw(void *clientData, void *elementRecord,
    Tk_Window tkwin, Drawable d, TTK_Box b, unsigned int state)
{
    PixmapData *pixmapData = clientData;
    PixmapElement *element = elementRecord;
    Tcl_Obj *imageObj = TTK_StateMapLookup(NULL, pixmapData->imageMap, state);
    TTK_Box parcel = b, imgbox = TTK_MakeBox(0, 0, 0, 0);
    Tk_Image image;


    /* We need to erase the background to accommodate transparent image parts */
    if (pixmapData->background == BACKGROUND_OPAQUE && element->backgroundObj) {
	Tk_3DBorder background = 
	    Tk_Get3DBorderFromObj(tkwin, element->backgroundObj);
	GC gc = Tk_3DBorderGC(tkwin, background, TK_3D_FLAT_GC);
	XFillRectangle(Tk_Display(tkwin), d, gc,
	    parcel.x, parcel.y, parcel.width, parcel.height);
    }

    image = TTK_UseImage(pixmapData->cache, tkwin, imageObj);

    if (!image)
	return;

    Tk_SizeOfImage(image, &imgbox.width, &imgbox.height);

    if (pixmapData->tiling == TILING_TILE) {
	TTK_Tile(tkwin, d, image, &imgbox, &parcel, pixmapData->border); 
    }

    if (pixmapData->tiling == TILING_FIXED) {
	/* FIX ME: this could perhaps benefit from an -anchor option */
	int x2 = parcel.x + parcel.width/2 - imgbox.width/2;
	int y2 = parcel.y + parcel.height/2 - imgbox.height/2;
	x2 = (x2 < parcel.x) ? parcel.x : x2;
	y2 = (y2 < parcel.y) ? parcel.y : y2;
	Tk_RedrawImage(image, 0, 0, b.width, b.height, d, x2, y2);
    }
}

static TTK_ElementSpec PixmapElementSpec = 
{
    TK_STYLE_VERSION_2,
    sizeof(PixmapElement),
    PixmapElementOptions,
    PixmapElementGeometry,
    PixmapElementDraw
};

/*
 * Element constructor for pixmap elements:
 */
static int 
CreatePixmapElement(
    Tcl_Interp *interp,
    void *clientData,
    TTK_Theme theme,
    const char *elementName,
    int objc, 
    Tcl_Obj *CONST objv[])
{
    Tk_OptionTable pixmapOptionTable = 
	Tk_CreateOptionTable(interp, PixmapOptionSpecs);
    PixmapData *pixmapData;

    pixmapData = (PixmapData*)ckalloc(sizeof(*pixmapData));
    memset(pixmapData, 0, sizeof(*pixmapData));
    pixmapData->interp = interp;
    pixmapData->cache = TTK_GetResourceCache(interp);

    /* ??? Check objc? */
    if (TCL_OK != Tk_SetOptions(interp, (ClientData)pixmapData,
	    pixmapOptionTable, objc, objv,
	    NULL/*tkwin*/, NULL/*savedOptions*/, NULL/*mask*/))
    {
	ckfree((ClientData)pixmapData);
	return TCL_ERROR;
    }

    if (TCL_OK != TTK_RegisterElementSpec(theme, elementName,
	&PixmapElementSpec, pixmapData))
    {
	Tcl_SetResult(interp, "Duplicate element name", TCL_STATIC);
	return TCL_ERROR;
    }

    /* @@@ pixmapData leaks */
    Tcl_SetObjResult(interp, Tcl_NewStringObj(elementName, -1));
    return TCL_OK;
}

/* ---------------------------------------------------------------------- */

int DLLEXPORT
PixmapTheme_Init(Tcl_Interp *interp)
{
    TTK_RegisterElementFactory(interp, "pixmap", CreatePixmapElement, NULL);
    Tcl_PkgProvide(interp, "tile::pixmap", TILE_VERSION);
    return TCL_OK;
}

/*EOF*/
