/* winTheme.c - Copyright (C) 2004 Pat Thoyts <patthoyts@users.sf.net>
 *
 * winTheme.c,v 1.17 2004/07/02 07:37:18 jenglish Exp
 */

#ifdef _MSC_VER
#define WIN32_LEAN_AND_MEAN
#endif

#include <windows.h>

#include <tk.h>

#ifdef NO_PRIVATE_HEADERS
#include <tkPlatDecls.h>
#include "gunk.h"
#else 
#include <tkWinInt.h>
#endif

#ifndef DFCS_HOT	/* Windows 98/Me, Windows 200/XP only */
#define DFCS_HOT 0
#endif

#include "tkTheme.h"

/*
 * BoxToRect --
 * 	Helper routine.  Returns a RECT data structure.
 */
static RECT
BoxToRect(TTK_Box b)
{
    RECT rc;
    rc.top = b.y;
    rc.left = b.x;
    rc.bottom = b.y + b.height;
    rc.right = b.x + b.width;
    return rc;
}

/*
 * ReliefToEdge --
 * 	Convert a Tk "relief" value into an Windows "edge" value.
 * 	NB: Caller must check for RELIEF_FLAT, which does not correspond
 * 	to any Win32 enum.
 */
static unsigned int
ReliefToEdge(int relief)
{
    switch (relief) {
	case TK_RELIEF_FLAT:   return BDR_RAISEDOUTER;	/* @@@ */
	case TK_RELIEF_RAISED: return EDGE_RAISED;
	case TK_RELIEF_SUNKEN: return EDGE_SUNKEN;
	case TK_RELIEF_GROOVE: return EDGE_ETCHED;
	case TK_RELIEF_SOLID:  return BDR_RAISEDOUTER;
	case TK_RELIEF_RIDGE:  return EDGE_BUMP;
	default:               return BDR_RAISEDOUTER;
    }
}

/* ---------------------------------------------------------------------- */

static TTK_StateTable checkbutton_statemap[] =
{
    { DFCS_CHECKED|DFCS_INACTIVE, TTK_STATE_SELECTED|TTK_STATE_DISABLED, 0 },
    { DFCS_CHECKED|DFCS_PUSHED,   TTK_STATE_SELECTED|TTK_STATE_PRESSED, 0 },
    { DFCS_CHECKED,	          TTK_STATE_SELECTED, 0 },
    { DFCS_INACTIVE,		  TTK_STATE_DISABLED, TTK_STATE_SELECTED },
    { DFCS_PUSHED,		  TTK_STATE_PRESSED, TTK_STATE_SELECTED},
    { 0,		  0, 0 }
};

static TTK_StateTable pushbutton_statemap[] =
{
    { DFCS_INACTIVE,	  TTK_STATE_DISABLED, 0 },
    { DFCS_PUSHED,	  TTK_STATE_PRESSED, 0 },
    { DFCS_HOT,		  TTK_STATE_ACTIVE, 0 },
    { 0,		  0, 0 }
};

static TTK_StateTable arrow_statemap[] = 
{
    { DFCS_INACTIVE,            TTK_STATE_DISABLED, 0 },
    { DFCS_PUSHED | DFCS_FLAT,  TTK_STATE_PRESSED,  0 },
    { 0, 0, 0 }
};

/* ---------------------------------------------------------------------- 
 * FrameControlElement --
 * 	General-purpose element for things drawn with DrawFrameControl
 */
typedef struct
{
    const char *name;
    int classId;
    int partId;
    int cxId;
    int cyId;
    TTK_StateTable *stateMap;
} FrameControlElementData;

static FrameControlElementData FrameControlElements[] =
{
    { "Checkbutton.indicator", 
	DFC_BUTTON, DFCS_BUTTONCHECK, SM_CYMENUCHECK, SM_CYMENUCHECK,
	checkbutton_statemap },
    { "Radiobutton.indicator",
    	DFC_BUTTON, DFCS_BUTTONRADIO, SM_CYMENUCHECK, SM_CYMENUCHECK,
	checkbutton_statemap },
    { "uparrow",
    	DFC_SCROLL, DFCS_SCROLLUP, SM_CXVSCROLL, SM_CYVSCROLL,
	arrow_statemap },
    { "downarrow", 
    	DFC_SCROLL, DFCS_SCROLLDOWN, SM_CXVSCROLL, SM_CYVSCROLL,
	arrow_statemap },
    { "leftarrow", 
	DFC_SCROLL, DFCS_SCROLLLEFT, SM_CXHSCROLL, SM_CYHSCROLL,
	arrow_statemap },
    { "rightarrow",
	DFC_SCROLL, DFCS_SCROLLRIGHT, SM_CXHSCROLL, SM_CYHSCROLL,
	arrow_statemap },

    { 0,0,0,0,0 }
};

/* ---------------------------------------------------------------------- */

static void
FrameControlElementGeometry(
    void *clientData, void *elementRecord,
    Tk_Window tkwin, int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    FrameControlElementData *elementData = clientData;
    *widthPtr = GetSystemMetrics(elementData->cxId);
    *heightPtr = GetSystemMetrics(elementData->cyId);
}

static void
FrameControlElementDraw(
    void *clientData, void *elementRecord,
    Tk_Window tkwin, Drawable d,
    TTK_Box b, unsigned int state)
{
    FrameControlElementData *elementData = clientData;
    RECT rc = BoxToRect(b);
    TkWinDCState dcState;
    HDC hdc = TkWinGetDrawableDC(Tk_Display(tkwin), d, &dcState);

    DrawFrameControl(hdc, &rc,
	elementData->classId,
	elementData->partId|TTK_StateTableLookup(elementData->stateMap, state));
    TkWinReleaseDrawableDC(d, hdc, &dcState);
}

static TTK_ElementSpec FrameControlElementSpec = 
{
    TK_STYLE_VERSION_2,
    sizeof(NullElement),
    NullElementOptions,
    FrameControlElementGeometry,
    FrameControlElementDraw
};

/*----------------------------------------------------------------------
 * +++ Border element implementation.
 */

typedef struct {
    Tcl_Obj	*reliefObj;
} BorderElement;

static TTK_ElementOptionSpec BorderElementOptions[] = {
    { "-relief",TK_OPTION_RELIEF,Tk_Offset(BorderElement,reliefObj), "flat" },
    {NULL}
};

static void
BorderElementGeometry(
    void *clientData, void *elementRecord,
    Tk_Window tkwin, int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    BorderElement *border = elementRecord;
    int relief;
    Tk_GetReliefFromObj(NULL, border->reliefObj, &relief);
    paddingPtr->left = paddingPtr->right = GetSystemMetrics(SM_CXEDGE);
    paddingPtr->top = paddingPtr->bottom = GetSystemMetrics(SM_CYEDGE);
}

static void
BorderElementDraw(
    void *clientData, void *elementRecord,
    Tk_Window tkwin, Drawable d,
    TTK_Box b, unsigned int state)
{
    BorderElement *border = elementRecord;
    RECT rc = BoxToRect(b);
    int relief = TK_RELIEF_FLAT;
    TkWinDCState dcState;
    HDC hdc;

    Tk_GetReliefFromObj(NULL, border->reliefObj, &relief);

    if (relief != TK_RELIEF_FLAT) {
	hdc = TkWinGetDrawableDC(Tk_Display(tkwin), d, &dcState);
        DrawEdge(hdc, &rc, ReliefToEdge(relief), BF_RECT);
	TkWinReleaseDrawableDC(d, hdc, &dcState);
    }
}

static TTK_ElementSpec BorderElementSpec =
{
    TK_STYLE_VERSION_2,
    sizeof(BorderElement),
    BorderElementOptions,
    BorderElementGeometry,
    BorderElementDraw
};

/*
 * Entry field borders:
 * Sunken border; also fill with window color.
 */

typedef struct
{
    Tcl_Obj	*backgroundObj;
} FieldElement;

static TTK_ElementOptionSpec FieldElementOptions[] =
{
    { "-fieldbackground", TK_OPTION_BORDER, 
    	Tk_Offset(FieldElement,backgroundObj), "white" },
    {NULL}
};

static void
FieldElementGeometry(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    paddingPtr->left = paddingPtr->right = GetSystemMetrics(SM_CXEDGE);
    paddingPtr->top = paddingPtr->bottom = GetSystemMetrics(SM_CYEDGE);
}

static void
FieldElementDraw(
    void *clientData, void *elementRecord, Tk_Window tkwin,
    Drawable d, TTK_Box b, unsigned int state)
{
    FieldElement *field = elementRecord;
    Tk_3DBorder bg = Tk_Get3DBorderFromObj(tkwin, field->backgroundObj);
    RECT rc = BoxToRect(b);
    TkWinDCState dcState;
    HDC hdc;

    Tk_Fill3DRectangle(
	tkwin, d, bg, b.x, b.y, b.width, b.height, 0, TK_RELIEF_FLAT);

    hdc = TkWinGetDrawableDC(Tk_Display(tkwin), d, &dcState);
    DrawEdge(hdc, &rc, EDGE_SUNKEN, BF_RECT);
    TkWinReleaseDrawableDC(d, hdc, &dcState);
}

static TTK_ElementSpec FieldElementSpec =
{
    TK_STYLE_VERSION_2,
    sizeof(FieldElement),
    FieldElementOptions,
    FieldElementGeometry,
    FieldElementDraw
};

/*
 * +++ Button borders:
 *	Drawn with DrawFrameControl instead of DrawEdge;
 *	Also draw default indicator and focus ring.
 */
typedef struct {
    Tcl_Obj	*reliefObj;
    Tcl_Obj	*highlightColorObj;
    Tcl_Obj	*defaultStateObj;
} ButtonBorderElement;

static TTK_ElementOptionSpec ButtonBorderElementOptions[] = {
    { "-relief",TK_OPTION_RELIEF,
	Tk_Offset(ButtonBorderElement,reliefObj), "flat" },
    { "-highlightcolor",TK_OPTION_COLOR,
	Tk_Offset(ButtonBorderElement,highlightColorObj), "black" },
    { "-default", TK_OPTION_ANY, 
	Tk_Offset(ButtonBorderElement,defaultStateObj), "disabled" },
    {NULL}
};

static void
ButtonBorderElementGeometry(
    void *clientData, void *elementRecord,
    Tk_Window tkwin, int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    ButtonBorderElement *bd = elementRecord;
    int relief = TK_RELIEF_RAISED;
    int defaultState = TTK_BUTTON_DEFAULT_DISABLED;
    short int cx, cy;

    Tk_GetReliefFromObj(NULL, bd->reliefObj, &relief);
    TTK_GetButtonDefaultStateFromObj(NULL, bd->defaultStateObj, &defaultState);
    cx = GetSystemMetrics(SM_CXEDGE);
    cy = GetSystemMetrics(SM_CYEDGE);

    /* Space for default indicator:
     */
    if (defaultState != TTK_BUTTON_DEFAULT_DISABLED) {
    	++cx; ++cy;
    }

    /* Space for focus ring:
     */
    cx += 2;
    cy += 2;

    *paddingPtr = TTK_MakePadding(cx,cy,cx,cy);
}

static void
ButtonBorderElementDraw(
    void *clientData, void *elementRecord,
    Tk_Window tkwin, Drawable d,
    TTK_Box b, unsigned int state)
{
    ButtonBorderElement *bd = elementRecord;
    int relief = TK_RELIEF_FLAT;
    int defaultState = TTK_BUTTON_DEFAULT_DISABLED;
    TkWinDCState dcState;
    HDC hdc;
    RECT rc;

    Tk_GetReliefFromObj(NULL, bd->reliefObj, &relief);
    TTK_GetButtonDefaultStateFromObj(NULL, bd->defaultStateObj, &defaultState);

    if (defaultState == TTK_BUTTON_DEFAULT_ACTIVE) {
	XColor *highlightColor =
	    Tk_GetColorFromObj(tkwin, bd->highlightColorObj);
	GC gc = Tk_GCForColor(highlightColor, d);
	XDrawRectangle(Tk_Display(tkwin), d, gc, b.x,b.y,b.width-1,b.height-1);
    }
    if (defaultState != TTK_BUTTON_DEFAULT_DISABLED) {
	++b.x; ++b.y; b.width -= 2; b.height -= 2;
    }

    hdc = TkWinGetDrawableDC(Tk_Display(tkwin), d, &dcState);

    rc = BoxToRect(b);
    DrawFrameControl(hdc, &rc,
	DFC_BUTTON,	/* classId */
	DFCS_BUTTONPUSH | TTK_StateTableLookup(pushbutton_statemap, state));

    /* Draw focus ring:
     */
    if (state & TTK_STATE_FOCUS) {
	short int borderWidth = 3;	/* @@@ Use GetSystemMetrics?*/
	rc = BoxToRect(TTK_PadBox(b, TTK_UniformPadding(borderWidth)));
    	DrawFocusRect(hdc, &rc);
    }
    TkWinReleaseDrawableDC(d, hdc, &dcState);
}

static TTK_ElementSpec ButtonBorderElementSpec =
{
    TK_STYLE_VERSION_2,
    sizeof(ButtonBorderElement),
    ButtonBorderElementOptions,
    ButtonBorderElementGeometry,
    ButtonBorderElementDraw
};


/*
 * Focus element --
 * Draw focus rectangle.
 */

static void FocusElementGeometry(
    void *clientData, void *elementRecord,
    Tk_Window tkwin, int *widthPtr, int *heightPtr, TTK_Padding *paddingPtr)
{
    *paddingPtr = TTK_UniformPadding(1);
}

static void
FocusElementDraw(
    void *clientData, void *elementRecord,
    Tk_Window tkwin, Drawable d, 
    TTK_Box b, unsigned int state)
{
    if (state & TTK_STATE_FOCUS) {
	RECT rc = BoxToRect(b);
	TkWinDCState dcState;
	HDC hdc = TkWinGetDrawableDC(Tk_Display(tkwin), d, &dcState);
    	DrawFocusRect(hdc, &rc);
	TkWinReleaseDrawableDC(d, hdc, &dcState);
    }
}

static TTK_ElementSpec FocusElementSpec =
{
    TK_STYLE_VERSION_2,
    sizeof(NullElement),
    NullElementOptions,
    FocusElementGeometry,
    FocusElementDraw
};

/* ---------------------------------------------------------------------- 
 * Layouts:
 */

TTK_BEGIN_LAYOUT(ButtonLayout)
    TTK_GROUP("Button.background", TTK_FILL_BOTH,
	TTK_GROUP("Button.border", TTK_FILL_BOTH,
	    TTK_GROUP("Button.padding", TTK_FILL_BOTH,
		TTK_NODE("Button.label", TTK_FILL_BOTH))))
TTK_END_LAYOUT

/* ---------------------------------------------------------------------- */

int WinTheme_Init(Tcl_Interp *interp)
{
    TTK_Theme themePtr, parentPtr;
    FrameControlElementData *fce = FrameControlElements;

    parentPtr = TTK_GetTheme(interp, "alt");
    themePtr = TTK_CreateTheme(interp, "winnative", parentPtr);
    if (!themePtr) {
        return TCL_ERROR;
    }

    TTK_RegisterElementSpec(themePtr, "border", &BorderElementSpec, NULL);
    TTK_RegisterElementSpec(themePtr, "Button.border", 
	&ButtonBorderElementSpec, NULL);
    TTK_RegisterElementSpec(themePtr, "field", &FieldElementSpec, NULL);
    TTK_RegisterElementSpec(themePtr, "focus", &FocusElementSpec, NULL);
    for (fce = FrameControlElements; fce->name != 0; ++fce) {
	TTK_RegisterElementSpec(themePtr, fce->name, 
		&FrameControlElementSpec, fce);
    }

    TTK_RegisterLayout(themePtr, "TButton", ButtonLayout);

    Tcl_PkgProvide(interp, "tile::theme::winnative", TILE_VERSION);
    return TCL_OK; 
}

